function [M,K] = SystemMatrices(nodcoor,elemnod,elemSubm,elemCrack,elemE,...
                                elemrho,elemA,elemI,elemD)
% SystemMatrices Establishes system stiffness and mass matrix
%   nodcoor: List of nodal coordinates [nnod x 2 matrix]
%   elemnod: List of element nodes [nelem x 2 matrix]
%   elemE: List of elements' Young's moduli
%   elemA: List of elements' cross-section area
%   elemI: List of elements' area moment of inertia
%   elemrho: List of elements' material density

nnod = size(nodcoor,1);
nelem = size(elemnod,1);

M = zeros(3*nnod,3*nnod);
K = zeros(3*nnod,3*nnod);

for i = 1:nelem                 % Loop over all elements
    
    % Global coordinates
    nod1 = elemnod(i,1);
    nod2 = elemnod(i,2);
    eft = [3*nod1-2, 3*nod1-1, 3*nod1, 3*nod2-2, 3*nod2-1, 3*nod2];
    
    % Local coordinates
    elemnodcoor = [nodcoor(nod1,:); nodcoor(nod2,:)];
    
    % Member's properties
    E = elemE(i);
    rho = elemrho(i);
    A = elemA(i);
    I = elemI(i);
    D = elemD(i);
    
    % Member submerged?
    subm = elemSubm(i);
    
    % Member cracked?
    crack = elemCrack(i);
    
    % Element matrices    
    [Me,Ke] = ElementMatrices(elemnodcoor,subm,crack,E,rho,A,I,D);
    
    % Assembly to global matrices
    for j = 1:6                             % Loop over all rows of el. matrix
        jj = eft(j);                        % jj: Global degree of freedom
        for k = 1:6                         % Loop over all columns of el. matrix
            kk = eft(k);                    % kk: Global degree of freedom
            M(jj,kk) = M(jj,kk) + Me(j,k);  
            K(jj,kk) = K(jj,kk) + Ke(j,k);  
        end
    end
end

