function [x_f,P_f] = kalman_filter(A,B,C,D,Q,R,P0,x0,p,d)

% DISCRETE-TIME STATE-SPACE MODEL:
% x_k+1 = A*x_k + B*p_k + w_k
% d_k = C*x_k + D*p_k + v_k
% Q:    process noise covariance [2*nDOF x 2*nDOF]
% R:    measurement noise covariance [2*nDOF x 2*nDOF]
% x0:   initial state estimate at t = 0 [2*nDOF x 1]
% P0:   initial error covariance at t = 0 [2*nDOF x 2*nDOF]
% N:    number of samples
% p:    applied forces [n_p x N]
% d:    data vector [n_d x N]

N = size(d,2);                  % Number of samples
nDOF = size(A,1)/2;             % Number of degrees of freedom
x_f = zeros(2*nDOF,N);          % Preallocation
P_f = zeros(2*nDOF,2*nDOF);     % Preallocation

for k = 1:N
	%	Time update
	if k==1
		x_f(:,k) = A*x0;
		P_f(:,:,k) = A*P0*A' + Q;
	else
		x_f(:,k) = A*x_f(:,k-1)+B*p(:,k-1);
		P_f(:,:,k) = A*P_f(:,:,k-1)*A' + Q;
	end
	
	%	Measurement update
	K_k = P_f(:,:,k)*C'*(C*P_f(:,:,k)*C'+R)^(-1);
	x_f(:,k) = x_f(:,k) + K_k*(d(:,k)-C*x_f(:,k)-D*p(:,k));
	P_f(:,:,1) = P_f(:,:,k) - K_k*C*P_f(:,:,k);
end

