% 2D jacket structure according to Vorpahl's UpWind Reference Jacket
% 312 DOF

clc
clear all
close all
tic

global eom f zeta omega k phase_JS u_c Cm Cd d f_wi df_wi phase_list V_hub...
       sigma1 nodcoor elemnod elemD elemSubm elemWind Kc

%% Input values

% Crack 'number'
cn = 1;                 % 1 to 8
cn_str = num2str(cn-1);

% Time span of solution
fs = 300;
dt = 1/fs;              % Sample size/time step
T = 1200;
t = 0:dt:T-dt;          % [s]
N = length(t);

% Node coordinates
nodcoor = [-6.000   0.000;...   % 1     -
           -6.000   0.500;...   % 2     -
           -5.967   1.499;...   % 3     -
           -5.939   2.373;...   % 4     -
           -5.788   7.001;...   % 5
           -5.636  11.630;...   % 6
           -5.485  16.258;...   % 7
           -5.333  20.886;...   % 8     -
           
            6.000   0.000;...   % 9     -
            6.000   0.500;...   % 10    -
            5.967   1.499;...   % 11    -
            5.939   2.373;...   % 12    -
            5.788   7.001;...   % 13
            5.636  11.630;...   % 14
            5.485  16.258;...   % 15
            5.333  20.886;...   % 16    -
            
           -5.205  24.809;...   % 17
           -5.077  28.732;...   % 18
           -4.948  32.655;...   % 19
           -4.820  36.578;...   % 20    -
           -4.711  39.903;...   % 21
           -4.603  43.228;...   % 22
           -4.494  46.553;...   % 23
           -4.385  49.878;...   % 24    -
           -4.293  52.696;...   % 25
           -4.201  55.515;...   % 26
           -4.108  58.333;...   % 27
           -4.016  61.151;...   % 28    -
           -4.000  61.650;...   % 29    -
           
            5.205  24.809;...   % 30
            5.077  28.732;...   % 31
            4.948  32.655;...   % 32
            4.820  36.578;...   % 33    -
            4.711  39.903;...   % 34
            4.603  43.228;...   % 35
            4.494  46.553;...   % 36
            4.385  49.878;...   % 37    -
            4.293  52.696;...   % 38
            4.201  55.515;...   % 39
            4.108  58.333;...   % 40
            4.016  61.151;...   % 41    -
            4.000  61.650;...   % 42    -
            
           -4.454   4.812;...   % 43
           -2.970   7.250;...   % 44
           -1.485   9.689;...   % 45
            4.454   4.812;...   % 46
            2.970   7.250;...   % 47
            1.485   9.689;...   % 48
            
            0.000  12.127;...   % 49    -
            
           -1.333  14.317;...   % 50
           -2.667  16.507;...   % 51
           -4.000  18.696;...   % 52
            1.333  14.317;...   % 53
            2.667  16.507;...   % 54
            4.000  18.696;...   % 55
            
           -4.000  22.947;...   % 56
           -2.667  25.008;...   % 57
           -1.333  27.068;...   % 58
            4.000  22.947;...   % 59
            2.667  25.008;...   % 60
            1.333  27.068;...   % 61
            
            0.000  29.129;...   % 62    -
            
           -1.205  30.991;...   % 63
           -2.410  32.854;...   % 64
           -3.615  34.716;...   % 65
            1.205  30.991;...   % 66
            2.410  32.854;...   % 67
            3.615  34.716;...   % 68
            
           -3.615  38.319;...   % 69
           -2.410  40.060;...   % 70
           -1.205  41.801;...   % 71
            3.615  38.319;...   % 72
            2.410  40.060;...   % 73
            1.205  41.801;...   % 74
            
            0.000  43.542;...   % 75    -
            
           -1.096  45.126;...   % 76
           -2.193  46.710;...   % 77
           -3.289  48.294;...   % 78
            1.096  45.126;...   % 79
            2.193  46.710;...   % 80
            3.289  48.294;...   % 81
            
           -3.289  51.349;...   % 82
           -2.193  52.820;...   % 83
           -1.096  54.291;...   % 84
            3.289  51.349;...   % 85
            2.193  52.820;...   % 86
            1.096  54.291;...   % 87
            
            0.000  55.762;...   % 88    -
            
           -1.004  57.109;...   % 89
           -2.008  58.457;...   % 90
           -3.012  59.804;...   % 91
            1.004  57.109;...   % 92
            2.008  58.457;...   % 93
            3.012  59.804;...   % 94
            
            0.000  61.650;...   % 95    -
            0.000  62.650;...   % 96    -
            0.000  73.650;...   % 97    -
            0.000  83.650;...   % 98    -
            0.000  95.650;...   % 99    -
            0.000 105.650;...   % 100   -
            0.000 115.650;...   % 101   -
            0.000 124.650;...   % 102   -
            0.000 129.650;...   % 103   -
            4.218  35.647];     % 104

nDOF = size(nodcoor,1)*3;

% Element nodes
elemnod = [ 1, 2;  2, 3;  3, 4;  4, 5;  5, 6;  6, 7;  7, 8;...  % Legs (1-14)
            9,10; 10,11; 11,12; 12,13; 13,14; 14,15; 15,16;...
            8,17; 17,18; 18,19; 19,20; 20,21; 21,22; 22,23; 23,24; 24,25;...
           25,26; 26,27; 27,28; 28,29;...   % Legs (15-40)
           16,30; 30,31; 31,32; 32,33; 33,34; 34,35; 35,36; 36,37; 37,38;... 
           38,39; 39,40; 40,41; 41,42;...
            4,43; 43,44; 44,45; 45,49;...                  % X-braces (41-104)
           12,46; 46,47; 47,48; 48,49;...
           49,50; 50,51; 51,52; 52, 8;...
           49,53; 53,54; 54,55; 55,16;...
            8,56; 56,57; 57,58; 58,62;...
           16,59; 59,60; 60,61; 61,62;...
           62,63; 63,64; 64,65; 65,20;...
           62,66; 66,67; 67,68; 68,104; 104,33;...
           20,69; 69,70; 70,71; 71,75;...
           33,72; 72,73; 73,74; 74,75;...
           75,76; 76,77; 77,78; 78,24;...
           75,79; 79,80; 80,81; 81,37;...
           24,82; 82,83; 83,84; 84,88;...
           37,85; 85,86; 86,87; 87,88;...
           88,89; 89,90; 90,91; 91,28;...
           88,92; 92,93; 93,94; 94,41;...
            3,11; 29,95; 95,42;...                         % Horizontal (105-107)
           95,96; 96,97; 97,98; 98,99; 99,100; 100,101; 101,102; 102,103];  
                                                           % Tower (108-115)

% Material properties
Es = 2.1e11;        % [N/m2]
Ec = 3e10;          % [N/m2]
rho_s = 7850;       % [kg/m3]
rho_c = 2000;       % [kg/m3]

elemE = [Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Es, Es, Es, Es, Es, Es, Es,...
         Es, Ec, Ec,...
         Es, Es, Es, Es, Es, Es, Es, Es]; 
     
elemrho = [rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s,...
           rho_s, rho_c, rho_c,...
           rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s, rho_s]; 

% Member properties

% Jacket
D1 = 0.8;
D2 = 1.2;
D3 = 1.2;
D4 = 1.2;
t1 = 0.02;
t2 = 0.05;
t3 = 0.035;
t4 = 0.04;
A1 = pi/4*(D1^2-(D1-2*t1)^2);
A2 = pi/4*(D2^2-(D2-2*t2)^2);
A3 = pi/4*(D3^2-(D3-2*t3)^2);
A4 = pi/4*(D4^2-(D4-2*t4)^2);
I1 = pi/64*(D1^4-(D1-2*t1)^4);
I2 = pi/64*(D2^4-(D2-2*t2)^4);
I3 = pi/64*(D3^4-(D3-2*t3)^4);
I4 = pi/64*(D4^4-(D4-2*t4)^4);

% Tower
% Dt = [5.600/1.95; 5.577/1.95; 5.318/1.95; 5.082/1.95; 4.800/1.95; 4.565/1.95;...
%       4.329/1.95; 4.118/1.95; 4.000/1.95];
% tt = [0.032/2; 0.032/2.3; 0.030/2; 0.028/2; 0.024/2; 0.022/2; 0.020/2;...
%       0.030/2; 0.030/2];
Dt = [5.600/1; 5.577/1; 5.318/1; 5.082/1; 4.800/1; 4.565/1; 4.329/1;... 
      4.118/1; 4.000/1];
tt = [0.032/1; 0.032/1; 0.030/1; 0.028/1; 0.024/1; 0.022/1; 0.020/1;...
      0.030/1; 0.030/1];
Dte = zeros(length(Dt)-1,1);
tte = zeros(length(Dt)-1,1);
At = zeros(length(Dt)-1,1);
It = zeros(length(Dt)-1,1);
for i = 2:length(Dt)
    Dte(i-1) = (Dt(i-1)+Dt(i))/2;
    tte(i-1) = (tt(i-1)+tt(i))/2;
    At(i-1) = pi/4*(Dte(i-1)^2-(Dte(i-1)-2*tte(i-1))^2);
    It(i-1) = pi/64*(Dte(i-1)^4-(Dte(i-1)-2*tte(i-1))^4);
end

% Transition piece
b_TP = 9.6;                 % [m]
h_TP = 4;                   % [m]
l_TP = 8;                   % [m]
I_TP = b_TP*h_TP^3/12;      % [m4]
m_TP = 666000;              % [kg]
A_TP = m_TP/(rho_c*l_TP);   % [m2]

elemA = [A2, A2, A2, A2, A2, A2, A2, A2, A2, A2, A2, A2, A2, A2,...
         A3, A3, A3, A3, A3, A3, A3, A3, A3, A3, A3, A3, A3,...
         A3, A3, A3, A3, A3, A3, A3, A3, A3, A3, A3, A3, A3,...
         A1, A1, A1, A1, A1, A1, A1, A1,...
         A1, A1, A1, A1, A1, A1, A1, A1,...
         A1, A1, A1, A1, A1, A1, A1, A1,...
         A1, A1, A1, A1, A1, A1, A1, A1, A1,...
         A1, A1, A1, A1, A1, A1, A1, A1,...
         A1, A1, A1, A1, A1, A1, A1, A1,...
         A1, A1, A1, A1, A1, A1, A1, A1,...
         A1, A1, A1, A1, A1, A1, A1, A1,...
         A1, A_TP/2, A_TP/2,...
         At(1), At(2), At(3), At(4), At(5), At(6), At(7), At(8)];
     
elemI = [I2, I2, I2, I2, I2, I2, I2, I2, I2, I2, I2, I2, I2, I2,...
         I3, I3, I3, I3, I3, I3, I3, I3, I3, I3, I3, I3, I3,...
         I3, I3, I3, I3, I3, I3, I3, I3, I3, I3, I3, I3, I3,...
         I1, I1, I1, I1, I1, I1, I1, I1,...
         I1, I1, I1, I1, I1, I1, I1, I1,...
         I1, I1, I1, I1, I1, I1, I1, I1,...
         I1, I1, I1, I1, I1, I1, I1, I1, I1,...
         I1, I1, I1, I1, I1, I1, I1, I1,...
         I1, I1, I1, I1, I1, I1, I1, I1,...
         I1, I1, I1, I1, I1, I1, I1, I1,...
         I1, I1, I1, I1, I1, I1, I1, I1,...
         I1, I_TP/2, I_TP/2,...
         It(1), It(2), It(3), It(4), It(5), It(6), It(7), It(8)];
     
elemD = [D2, D2, D2, D2, D2, D2, D2, D2, D2, D2, D2, D2, D2, D2,...
         D3, D3, D3, D3, D3, D3, D3, D3, D3, D3, D3, D3, D3,...
         D3, D3, D3, D3, D3, D3, D3, D3, D3, D3, D3, D3, D3,...
         D1, D1, D1, D1, D1, D1, D1, D1,...
         D1, D1, D1, D1, D1, D1, D1, D1,...
         D1, D1, D1, D1, D1, D1, D1, D1,...
         D1, D1, D1, D1, D1, D1, D1, D1, D1,...
         D1, D1, D1, D1, D1, D1, D1, D1,...
         D1, D1, D1, D1, D1, D1, D1, D1,...
         D1, D1, D1, D1, D1, D1, D1, D1,...
         D1, D1, D1, D1, D1, D1, D1, D1,...
         D1, 1, 1,...
         Dt(1), Dt(2), Dt(3), Dt(4), Dt(5), Dt(6), Dt(7), Dt(8)];

elemSubm = ones(size(elemnod,1),1);  % Element submerged? 0=false, 1=true
subm = [23:27,36:40,90:116];         % Insert elements that are NOT submerged
elemSubm(subm) = 0;                    

elemWind = zeros(size(elemnod,1),1); % Element subject to wind load? 0=false, 1=true
elemWind(109:116) = 1;               % Insert elements that are subject to wind 

elemCrack = zeros(size(elemnod,1),1);  % Element cracked? 0=false, 1=true
elemCrack(73) = 1;

Kc = [0, 0.00157821, 0.00871166, 0.0215477, 0.0320845, 0.0438089, 0.0502119,... 
      0.0560688]; % Crack depth (0.01 m 0.02 m 0.05 m 0.10 m 0.20 m 0.30 m 0.40 m)
Kc = Kc(cn); 



%% System Matrices

% System mass and stiffness matrices M and K
[M,K] = SystemMatrices(nodcoor,elemnod,elemSubm,elemCrack,elemE,elemrho,...
                       elemA,elemI,elemD);

% Check for singularity of stiffness matrix separately in Mathematica

% Boundary conditions

% Top mass
% m_RNA = 350000/10;                    % Top mass in [kg] (NREL 5 MW 350t RNA)
m_RNA = 350000/1.5;
m_R = 110000;                           % Rotor mass: 110,000 kg
m_NA = 240000;                          % Nacelle mass: 240,000 kg
r_R = 63;                               % Rotor diameter: 63 m
l_NA = 14;                              % Length of nacelle: 14 m
h_NA = 3.5;                             % Height of nacelle: 3.5 m
I_R = m_R*r_R^2/4;
I_NA = 1/12*m_NA*(l_NA^2+h_NA^2);
I_RNA = I_R + I_NA;
% m_top = 8000;
M(307,307) = M(307,307) + m_RNA;
M(308,308) = M(308,308) + m_RNA;
M(309,309) = M(309,309) + I_RNA/3;

% Additional tower point masses
m_t1 = 1900;
% m_t1 = 500;
M(283,283) = M(283,283) + m_t1;
M(284,284) = M(284,284) + m_t1;

m_t2 = 1400;
% m_t2 = 300;
M(295,295) = M(295,295) + m_t2;
M(296,296) = M(296,296) + m_t2;

m_t3 = 1000;
% m_t3 = 200;
M(307,307) = M(307,307) + m_t3;
M(308,308) = M(308,308) + m_t3;

% Soil springs
k_x = 6.7821e+08;                     % Soil stiffness in x direction
k_xr = 1.7969e+09;                    % Soil stifness coupling between x and r
k_z = 2.1940e+09;                     % Soil stiffness in z direction
k_r = 1.3061e+10;                     % Soil stiffness in rotational direction

K(1,1) = K(1,1) + k_x;                % Leg 1
K(2,2) = K(2,2) + k_z;
K(3,3) = K(3,3) + k_r;
K(1,3) = K(1,3) + k_xr;
K(3,1) = K(3,1) + k_xr;

K(25,25) = K(25,25) + k_x;            % Leg 2
K(26,26) = K(26,26) + k_z;
K(27,27) = K(27,27) + k_r;
K(25,27) = K(25,27) + k_xr;
K(27,25) = K(27,25) + k_xr;

% Rayleigh damping matrix
% C = zeros(size(M,1),size(M,1));
C = 0.0001*M + 0.0001*K;

% Soil dashpots
c_x = 8.8800e+07;                     % Damping value in x direction
c_z = 2.8727e+08;                     % Damping value in z direction
c_r = 1.7101e+09;                     % Damping value in rotational direction
c_xr = 2.3527e+08;

C(1,1) = C(1,1) + c_x;                % Leg 1
C(2,2) = C(2,2) + c_z;
C(3,3) = C(3,3) + c_r;
C(1,3) = C(1,3) + c_xr;
C(3,1) = C(3,1) + c_xr;

C(25,25) = C(25,25) + c_x;            % Leg 2
C(26,26) = C(26,26) + c_z;
C(27,27) = C(27,27) + c_r;
C(25,27) = C(25,27) + c_xr;
C(27,25) = C(27,25) + c_xr;

% Mass matrix
eom.M = M;
Mi = M^(-1);
eom.Mi = Mi;

% Stiffness matrix     
eom.K = K;

% Damping matrix
eom.C = C;

% Matrix multiplications
eom.MK = -eom.Mi * eom.K;
eom.MC = -eom.Mi * eom.C;



%% Hydrodynamic load

% Irregular wave properties
Hs = 6;                 % Significant wave height [m]
Tp = 10;                % Spectral peak period [s]
u_c = 1;                % Current velocity [m/s]
Cm = 1.6;               % Inertia coefficient
Cd = 0.8;               % Drag coefficient
d = 49.878;             % Water depth [m]
fp = 1/Tp;              % Spectral peak frequency [Hz]

% Frequency band
df = 0.001;
f = df:df:0.5;
% phase_JS = 2*pi*rand(length(f),1);      % Random phases
load('phase_JS')

S_JS = zeros(length(f),1);
zeta = zeros(length(f),1);
omega = zeros(length(f),1);
k = zeros(length(f),1);
eta = 0;


% JONSWAP spectrum
if Tp/sqrt(Hs) <= 3.6
    gamma = 5; 
elseif Tp/sqrt(Hs) > 3.6 && Tp/sqrt(Hs) <= 5
    gamma = exp(5.75-1.15*Tp/sqrt(Hs));
else
    gamma = 1;
end

for i = 1:length(f)

    if f(i) <= fp
        sigma = 0.07;
    else
        sigma = 0.09;
    end

    S_JS(i) = 0.3125*Hs^2*Tp*(f(i)/fp)^(-5)*(1-0.287*log(gamma))...
              *exp(-1.25*(f(i)/fp)^(-4))*gamma^exp(-0.5*((f(i)-fp)/(sigma*fp))^2);

    zeta(i) = sqrt(2*S_JS(i)*df);   % Wave amplitude [m]
    omega(i) = 2*pi*f(i);           % Wave frequency [rad/s]
    k(i) = kSolve(omega(i),d);      % Wave number [rad/m]

    % Wave elevation
    eta = eta + zeta(i)*sin(omega(i)*t+phase_JS(i));
end



%% Wind load

% Wind properties
V_hub = 12;                         % Mean wind speed at hub height
Iref = 0.12;                        % Turbulence intensity
sigma1 = Iref*(0.75*V_hub+5.6);     % Standard deviation (invariant with height)

% Frequency band
df_wi = 0.001;
f_wi = df_wi:df_wi:0.5;
% phase_list = 2*pi*rand(10,length(f_wi));
load('phase_list')

S_KM = zeros(length(f_wi),1);
amp_wi = zeros(length(f_wi),1);
omega_wi = zeros(length(f_wi),1);
V_var = 0;

z_hub = 129.65;
if z_hub-d < 60
    Lk = 5.67*(z_hub-d);            % Integral scale parameter Lk
else
    Lk = 8.1*42;
end
I = sigma1/V_hub;

for j = 1:length(f_wi)
    S_KM(j) = 4*I^2*V_hub*Lk/((1+6*f_wi(j)*Lk/V_hub)^(5/3)); % Kaimal spectrum
    amp_wi(j) = sqrt(2*S_KM(j)*df_wi);                       % Wind amplitude [m/s]
    omega_wi(j) = 2*pi*f_wi(j);                              % Wind frequency [rad/s]
    V_var = V_var + amp_wi(j)*sin(omega_wi(j)*t+phase_list(1,j));
end

V_mean = V_hub;           % Mean wind speed at hub height
V = V_mean + V_var;



%% Force vector

f_hydro = zeros(nDOF,N);
f_wind = zeros(nDOF,N);
f_total = zeros(nDOF,N);
for w = 1:N
    v = t(w);
    f_hydro(:,w) = Hydro_Morison_irr_lin(v);
    f_wind(:,w) = Wind_Morison_lin(v);
    f_total(:,w) = f_hydro(:,w) + f_wind(:,w);
end

% Add harmonic force with w=0.305 Hz
% A_h = 50;           % [N]
% f_h = 0.305;        % [Hz]
% omega_h = 2*pi*f_h; % [rad/s]
% f_harm = A_h*sin(omega_h*t);
% f_total(70,:) = f_total(70,:) + f_harm;

% Wave velocities and accelerations
u_x1 = 0;
u_z1 = 0;
u_x2 = 0;
u_z2 = 0;
a_x1 = 0;
a_z1 = 0;
a_x2 = 0;
a_z2 = 0;

x1 = -4.385;
z1 = 49.878;
for l = 1:length(f)
    u_x1 = u_x1 + omega(l)*zeta(l)*cosh(k(l)*z1)/sinh(k(l)*d)...
                  *sin(omega(l)*t-k(l)*x1+phase_JS(l));
    u_z1 = u_z1 + omega(l)*zeta(l)*sinh(k(l)*z1)/sinh(k(l)*d)...
                  *cos(omega(l)*t-k(l)*x1+phase_JS(l));
    a_x1 = a_x1 + omega(l)^2*zeta(l)*cosh(k(l)*z1)/sinh(k(l)*d)...
                  *cos(omega(l)*t-k(l)*x1+phase_JS(l));
    a_z1 = a_z1 - omega(l)^2*zeta(l)*sinh(k(l)*z1)/sinh(k(l)*d)...
                  *sin(omega(l)*t-k(l)*x1+phase_JS(l));
end



%% Solution

% 1) Initialise solution
n_p = nDOF;                 % Number of forces
n_d = nDOF;                 % Number of observations
x0 = zeros(2*nDOF,1);       % Initial conditions

% Selection matrices
S_v = zeros(n_d,nDOF);      % Velocity
S_d = zeros(n_d,nDOF);      % Displacements
S_p = eye(nDOF);            % Forces
S_a = eye(n_d,nDOF);        % Accelerations (all degrees of freedom)


% 2) Construct the (discrete-time) state-space matrices A, B, C and D
% System matrix A
A_c = [zeros(nDOF,nDOF) eye(nDOF); -Mi*K -Mi*C];
A = expm(A_c*dt);

% Input matrix B
B_c = [zeros(nDOF,n_p); Mi*S_p];
B = (A-eye(2*nDOF))*A_c^(-1)*B_c;

% Output matrix G
G = [S_d-S_a*Mi*K S_v-S_a*Mi*C];

% Direct transmission matrix J
J = S_a*Mi*S_p;

% Force matrix
p = f_total;

% State matrices
state.A = A;
state.B = B;
state.G = G;
state.J = J;


% 3) Solve
x_k = zeros(2*nDOF,N);      % Preallocation
for time = 1:N
	%	Time update
	if time==1
		x_k(:,time) = A*x0;
    else
		x_k(:,time) = A*x_k(:,time-1)+B*p(:,time-1);
	end
end


% 4) Response determination
n_d = nDOF;                          % Number of output positions
% Displacements
S_d1 = eye(n_d,nDOF);
S_v1 = zeros(n_d,nDOF);
S_a1 = zeros(n_d,nDOF);
G1 = [S_d1-S_a1*Mi*K S_v1-S_a1*Mi*C];
J1 = S_a1*Mi*S_p;
u_k = G1*x_k+J1*p;                   % Displacements [n_d x N], here [nDOF x N]

% Velocities
S_d2 = zeros(n_d,nDOF);
S_v2 = eye(n_d,nDOF);
S_a2 = zeros(n_d,nDOF);
G2 = [S_d2-S_a2*Mi*K S_v2-S_a2*Mi*C];
J2 = S_a2*Mi*S_p;
v_k = G2*x_k+J2*p;                   % Velocities [n_d x N], here [nDOF x N]

% Accelerations
S_d3 = zeros(n_d,nDOF);
S_v3 = zeros(n_d,nDOF);
S_a3 = eye(n_d,nDOF);
G3 = [S_d3-S_a3*Mi*K S_v3-S_a3*Mi*C];
J3 = S_a3*Mi*S_p;
a_k = G3*x_k+J3*p;                   % Accelerations [n_d x N], here [nDOF x N]

% 4) Save
dir = ['W:\Master Thesis\UpWind Jacket\Response\Results\Jacket 2\Cracked ' cn_str '\300Hz'];
cd(dir);

save('eom.mat','eom')
save('state.mat','state');
save('u_k.mat','u_k');
save('v_k.mat','v_k');
save('a_k.mat','a_k');
save('t.mat','t');
save('p.mat','p');
save('eta.mat','eta');
save('V.mat','V');

toc

